#![warn(missing_docs)]

//! # golgi
//!
//! _The Golgi complex (aka. Golgi apparatus or Golgi body) packages proteins into membrane-bound vesicles inside the cell before the vesicles are sent to their destination._
//!
//! -----
//!
//! Golgi is an experimental Scuttlebutt client which uses the [kuska-ssb](https://github.com/Kuska-ssb) libraries and aims to provide a high-level API for interacting with an sbot instance. Development efforts are currently oriented towards [go-sbot](https://github.com/cryptoscope/ssb) interoperability.
//!
//! ## Example Usage
//!
//! ```rust
//! use golgi::GolgiError;
//! use golgi::sbot::Sbot;
//!
//! pub async fn run() -> Result<(), GolgiError> {
//!     let mut sbot_client = Sbot::init(None, None).await?;
//!
//!     let id = sbot_client.whoami().await?;
//!     println!("{}", id);
//!
//!     Ok(())
//! }
//! ```

pub mod error;
pub mod messages;
pub mod sbot;
pub mod utils;

pub use crate::error::GolgiError;
